%macro CPSLMRPT(slmpath=,
                environmentDBPath=,
                shareService=,
                shareUser=,
                sharePassword=,
                reportid=,
                report=,
                action=SERVICE_LEVEL,
                outputlocationPath=,
                contract=,
                contractID=,
                textEncoding=,
                textTermStr=,
                _rc=);
 %*+----------------------------------------------------------------+
   | Copyright (c) 2004 by SAS Institute Inc., Cary, NC 27512-8000
   | Name:     CPSLMRPT.SAS
   | Support:  CES
   | Product:  SAS IT Service Level Management
   | Language: SAS
   | Purpose:  Run Reports
   |
   | Parameters:
   |
   |   Name              Description
   |  slmpath            path to the SLM software
   |  environmentDBPath  path to the SLM environment (assigned to SLM
   |                     libref)
   |  shareService       Service name (or port number) for SHARE
   |  shareUser          Userid for authentication to SHARE
   |  sharePassword      Password for authentication to SHARE
   |  reportID           ID of the report in the specified SLM environment
   |  report             Name of the reportt (easier for folks to use)
   |  action             SERVICE_LEVEL (default) | BASELINE
   |  outputlocationPath root of where to write out the reports. (for BASELINE)
   |  contract           name of contract (for BASELINE)
   |  contractid         if of contract (for BASELINE)
   |  textEncoding       [optional] encoding to use when creating HTML text
   |                     See the SAS National Language Users Guide
   |  textTermStr        [optional] character to use at line end of text files
   |                     See the SAS National Language Users Guide
   |  _rc                [optional] Name of macro var to contain RC
   |
   | Notes:
   |    The shareService= parameter is optional, and is only used if
   |    a SAS/SHARE server is used to access the environment database.
   |    The shareUser= and sharePassword= parameters are also optional
   |    and are specified for purposes of authentication to the SHARE
   |    server.
   |
   | end
   +----------------------------------------------------------------+;

   options compress=yes obs=max firstobs=1 nofmterr s=0 s2=0 user=WORK
           validvarname = UPCASE;
   %if %substr(&sysver,1,1) eq 9 %then %do;
      options noquotelenmax;
   %end;

   %******************************* ;
   %* Macro variable declarations * ;
   %******************************* ;
   %global _SLMrc;            %* ITSLM global return code variable ;
   %global SLMDEBUG;          %* ITSLM global debug flag ;
   %global _CPSLMRPTrc;       %* return code variable for this macro ;
   %global cppmrc;            %* ITMS Core global return code variable ;
   %global cproot;            %* ITMS Core software root ;
   %global cpsow;             %* ITMS Core needs this to be global ;

   %local  accessString;      %* System-specific readonly libname option ;

   %local  _action;             %* For upcase comparison ;
   %local  _actionParm;         %* For SCL invocation ;
   %local  _reportParm;         %* For SCL invocation ;
   %local  _envPathParm;        %* For SCL invocation ;
   %local  _shareServiceParm;   %* For SCL invocation ;
   %local  _shareUserParm;      %* For SCL invocation ;
   %local  _sharePasswordParm;  %* For SCL invocation ;
   %local  _debugParm;          %* For SCL invocation ;
   %local  _windowPrefix;       %* For SCL invocation ;
   %local  _windowRestore;      %* For SCL invocation ;
   %local  _contractParm;       %* For SCL invocation ;
   %local  _outputLocationParm; %* For SCL invocation ;
   %local  _textEncodingParm;   %* For SCL invocation ;
   %local  _textTermStrParm;    %* For SCL invocation ;
   %local  _parmRC;             %* _rc is being overwritten ;

   %local  CPSLMRPT_notes;      %* save and reset ;
   %local  CPSLMRPT_source;     %* save and reset ;
   %local  CPSLMRPT_source2;    %* save and reset ;
   %local  CPSLMRPT_mprint;     %* save and reset ;
   %local  CPSLMRPT_mlogic;     %* save and reset ;
   %local  CPSLMRPT_macrogen;   %* save and reset ;
   %local  CPSLMRPT_symbolgen;  %* save and reset ;
   %local  CPSLMRPT_errorabend; %* save and reset ;
   %local  CPSLMRPT_fmtsearch;  %* save and reset ;

   %******************************************************************* ;
   %* Save off some option settings to keep the noise down in the log * ;
   %******************************************************************* ;
   %let CPSLMRPT_notes      = %SYSFUNC(getoption(notes));
   %let CPSLMRPT_source     = %SYSFUNC(getoption(source));
   %let CPSLMRPT_source2    = %SYSFUNC(getoption(source2));
   %let CPSLMRPT_mprint     = %SYSFUNC(getoption(mprint));
   %let CPSLMRPT_mlogic     = %SYSFUNC(getoption(mlogic));
   %let CPSLMRPT_macrogen   = %SYSFUNC(getoption(macrogen));
   %let CPSLMRPT_symbolgen  = %SYSFUNC(getoption(symbolgen));
   %let CPSLMRPT_errorabend = %SYSFUNC(getoption(errorabend));
   %let CPSLMRPT_fmtsearch  = %SYSFUNC(getoption(fmtsearch));

   %******************************************* ;
   %* Debugging options and return code stuff * ;
   %******************************************* ;
   %if &SLMDEBUG eq 1 %then %do;
      options notes source source2 mprint mlogic macrogen symbolgen;
   %end;
   %else %do;
      options nonotes nosource nosource2 nomprint nomlogic nomacrogen
              nosymbolgen;
   %end;

   %let _CPSLMRPTrc = 99;   %* Assume a bad RC, let SCL give us a good RC;
   %if %length(&_rc) eq 0 %then %do;
       %local RC;
       %let _rc = RC;
   %end;
   %else %do;
        %let _parmRC = &_rc;
        %let &_rc = &_CPSLMRPTrc;
   %end;

   %********************************* ;
   %* Determine host-specific stuff * ;
   %********************************* ;
   %if %cphsttyp eq MVS %then %do;
       %let accessString = %str(disp=shr);
   %end;
   %else %do;
       %let accessString = %str(access=readonly);
   %end;

   %****************** ;
   %* Internal macro * ;
   %****************** ;
   %macro _releaseLib(name);
       %if %sysfunc(libref(&name)) eq 0 %then %do;
           libname &name clear;
       %end;
   %mend _releaseLib;

   %***************************** ;
   %* Set the ACTION= parameter *  ;
   %***************************** ;
   %let _action = %UPCASE(&action);
   %if (&_action ne SERVICE_LEVEL) and
       (&_action ne BASELINE) %then %do;
       %let _SLMrc= 8;
       %put ERROR:(&SYSMACRONAME)  SLM2001 Invalid action= parameter
value specified: &action;
       %goto EXIT;
   %end;
   %else %do;
       %let _actionParm = ACTION=&_action;
   %end;

   %****************************************************** ;
   %* If action is for a report specification object:    * ;
   %* Validate the REPORTID= and REPORT= parameters.     * ;
   %* We must have one of these parameters.  The SCL     * ;
   %* code will handle verifying that whatever is given  * ;
   %* is valid.  Here we just want to make sure we have  * ;
   %* exactly one of these parameters specified.         * ;
   %****************************************************** ;
   %if (&_action eq SERVICE_LEVEL) %then %do;
      %if ((%length(&reportID) eq 0) and
           (%length(&report) eq 0)) %then %do;
          %let _SLMrc= 8;
          %put ERROR:(&SYSMACRONAME)  SLM2003 Neither reportid= or report=
parameters were specified.;
          %goto EXIT;
      %end;

      %if ((%length(&reportID) gt 0) and
           (%length(&report) gt 0)) %then %do;
          %let _SLMrc= 8;
          %put ERROR:(&SYSMACRONAME)  SLM2004 Both reportid= or report=
parameters were specified.;
          %goto EXIT;
      %end;

      %if (%length(&reportID) gt 0) %then %do;
          %let _reportParm = REPORTID=&reportID;
      %end;

      %if (%length(&report) gt 0) %then %do;
          %let _reportParm = REPORT='&report';
      %end;

      %*********************************** ;
      %* Set the TEXTENCODING= parameter * ;
      %*********************************** ;
       %if (%length(&textEncoding) gt 0) %then %do;
          %let _textEncodingParm = ENCODING=&textEncoding;
      %end;

      %********************************** ;
      %* Set the TEXTTERMSTR= parameter * ;
      %********************************** ;
      %if (%length(&textTermStr) gt 0) %then %do;
          %let _textTermStrParm = TERMSTR=&textTermStr;
      %end;
   %end;

   %********************************************************** ;
   %* For BASELINE reports we need a location and a contract * ;
   %* Validate the CONTRACTID= and CONTRACT= parameters.     * ;
   %* We must have one of these parameters.  The SCL         * ;
   %* code will handle verifying that whatever is given      * ;
   %* is valid.  Here we just want to make sure we have      * ;
   %* exactly one of these parameters specified.             * ;
   %********************************************************** ;
   %else %if (&_action eq BASELINE) %then %do;
      %if (%length(outputLocation) eq 0) %then %do;
          %let _SLMrc= 8;
          %put ERROR:(&SYSMACRONAME)  SLM2003 No outputLocation
specified for baseline report.;
          %goto EXIT;
      %end;
      %else %do;
          %let _outputLocationParm =
             OUTPUTLOCATION=%str(%'&outputLocationPath%');
      %end;

      %if ((%length(&contractID) eq 0) and
           (%length(&contract) eq 0)) %then %do;
          %let _SLMrc= 8;
          %put ERROR:(&SYSMACRONAME)  SLM2003 Neither contractid= or
contract= parameters were specified.;
          %goto EXIT;
      %end;

      %if ((%length(&contractID) gt 0) and
           (%length(&contract) gt 0)) %then %do;
          %let _SLMrc= 8;
          %put ERROR:(&SYSMACRONAME)  SLM2004 Both contractid= or
contract= parameters were specified.;
          %goto EXIT;
      %end;

      %if (%length(&contractID) gt 0) %then %do;
          %let _contractParm = CONTRACTID=&contractID;
      %end;

      %if (%length(&contract) gt 0) %then %do;
          %let _contractParm = CONTRACT='&contract';
      %end;
   %end;

   %put NOTE:(&SYSMACRONAME)  SLM2000 SAS IT Service Level Management
initialization in progress.;

   %************************************** ;
   %* Massage the SLM environment parm.  * ;
   %* Be sure the pathname is not blank. * ;
   %************************************** ;
   %if (%length(&environmentDBPath) le 0) and
        (%length(&shareService) le 0) %then %do;
       %let _SLMrc = 8;
       %put ERROR:(&SYSMACRONAME)  SLM2002 Parameter EnvironmentDBPath
not specified or zero length.;
       %goto EXIT;
   %end;
   %let _envPathParm = ENVIRONMENT=%str(%'&environmentDBPath%');

   %****************************************;
   %* Add SHARE-related parameters, if any *;
   %****************************************;
   %if (%length(&shareService) gt 0) %then %do;
      %let _shareServiceParm = SHARESERVICE=&shareService;
      %if (%length(&shareUser) gt 0) %then
         %let _shareUserParm = SHAREUSER=%str(%'&shareUser%');
      %if (%length(&sharePassword) gt 0) %then
         %let _sharePasswordParm = SHAREPASSWORD=%str(%'&sharePassword%');
   %end;

   %******************************************* ;
   %* Allocate SLMLIB to access the software. * ;
   %******************************************* ;
   %cpslmlib(slmpath=&slmpath,_rc=_SLMrc);
   %if (&_SLMrc ne 0) %then %do ;
       %put ERROR:(&SYSMACRONAME)  SLM2006 Allocation failed for
&slmpath;
       %goto EXIT;
   %end;

   %************************************************************* ;
   %* Set the AF DEBUG= parm based upon the setting of SLMDEBUG * ;
   %************************************************************* ;
   %if &slmdebug eq 1 and &SYSENV eq FORE %then %do;
      %let _debugParm = YES;
   %end;
   %else %do;
      %let _debugParm = NO;
   %end;

   %********************************* ;
   %* Turn off confounding windows  * ;
   %********************************* ;
   %if %cphsttyp eq WIN %then %do;
       %local _dmsexp;
       proc sql noprint;
          select setting into :_dmsexp from dictionary.options
             where optname='DMSEXP';
       quit;  run;
       %if &_dmsexp eq DMSEXP %then %do;
          %let _windowPrefix =
            %str( ODSRESULTS; CAN; NEXT 'EXPLORER'; CAN; );
          %let _windowRestore = %str( ODSRESULTS; EXPLORER );
       %end;
    %end;

   %**************************************************** ;
   %* call the SCL code (return code is set in _SLMRC) * ;
   %**************************************************** ;
   dm "&_windowPrefix afa c=slmlib.slmenv.slmbatch.scl
       title='IT Service Level Management'
       aws=YES &_actionParm &_reportParm &_envPathParm
           &_outputLocationParm &_contractParm
           &_textEncodingParm   &_textTermStrParm
           &_shareServiceParm   &_shareUserParm
           &_sharePasswordParm  debug=&_debugParm" afa;

%EXIT:
   %if "&_windowRestore" ne "" %then %do;
       dm "&_windowRestore";
       %end;

   %_releaseLib(SLM)
   %_releaseLib(SLMLIB)
   %************************************************************** ;
   %* clear any librefs related to the contract PDB or ITMS Core * ;
   %************************************************************** ;
   %_releaseLib(PGMLIB)
   %_releaseLib(SITELIB)
   %_releaseLib(DETAIL)
   %_releaseLib(DAY)
   %_releaseLib(WEEK)
   %_releaseLib(MONTH)
   %_releaseLib(YEAR)
   %_releaseLib(PDBWORK)
   %_releaseLib(ADMIN)
   %_releaseLib(COLLECT)
   %_releaseLib(DICTLIB)

   %******************************************** ;
   %* Populate user return code macro variable * ;
   %******************************************** ;
   %if %eval(&_SLMrc gt .) %then %do;
       %let _CPSLMRPTrc = &_SLMrc;
   %end;
   %else %do;
       %let _SLMrc = &_CPSLMRPTrc;
   %end;
   %if %length(&_parmRC) gt 0 %then %do;
       %let &_parmRC = &_CPSLMRPTrc;
   %end;
   %put NOTE:(&SYSMACRONAME)  SLM2008 SAS IT Service Level Management ended
with return code &_CPSLMRPTrc;

   %***************** ;
   %* Reset options * ;
   %***************** ;
   options &CPSLMRPT_notes  &CPSLMRPT_source  &CPSLMRPT_source2
           &CPSLMRPT_mprint &CPSLMRPT_mlogic  &CPSLMRPT_macrogen
           &CPSLMRPT_symbolgen fmtsearch=&CPSLMRPT_fmtsearch;
%mend CPSLMRPT;
